unit ColorDisplay;



interface

uses windows,controls,classes, Graphics;


type TColorDisplaydir = (cbHor,cbVert);

     TColorSquare = record
                      x : integer;
                      y : integer;
                      Color : TColor;
                    end;

TColorSelect = procedure(sender : TObject; Index: integer; var color : TColor) of object;

TColorDisplay = class(TCustomControl)
     private
       FDirection  : TColorDisplayDir;
       FBorderWidth: byte;
       FBorderLight: LongInt;
       FBorderDark : LongInt;
       FSquareSize : byte;
       FColorPalette: TList;
       FSelectedColor: TColor;
       FOnSelect: TColorSelect;
       procedure SetDirection(CbDir : TColorDisplaydir);
       procedure SetDimensions;
       function GetColorSquare(Index : word) : TColorSquare;
       function GetColorPaletteCount: integer;
       procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
     protected
       procedure Paint; override;
       procedure SetSquareSize(Size : Byte);
       procedure SetBorderWidth(W : Byte);
       procedure SetBorderLight(C : LongInt);
       procedure SetBorderDark(C: LongInt);
     public
       constructor Create(AOwner:TComponent); override;
       destructor Destroy; override;
       procedure AddItem(const Value: TColor);
       procedure SetItem(const Index: integer; NewColor: TColor);
       procedure Clear;
     published
       property OnSelect: TColorSelect read FOnSelect write FOnSelect;
       property Direction: TColorDisplayDir read FDirection write SetDirection default CbHor;
       property SelectedColor: TColor read FSelectedColor;
       property ColorCount: integer read GetColorPaletteCount;
       property SquareSize : byte read FSquareSize write SetSquareSize default 10;
       property BorderWidth;
       property BorderLight : longInt read FBorderLight write SetborderLight default $ffffff;
       property BorderDark  : LongInt read FBorderDark write SetBorderDark default $0;
       property Visible;
       property Enabled;
  end;



procedure Register;

implementation

procedure Register;
begin
 RegisterComponents('system',[TColorDisplay]);
end;



procedure TColorDisplay.SetDimensions;
var xx,yy : integer;
begin
 yy := FcolorPalette.Count*FSquareSize+ 2*BorderWidth + 1;     //add border thickness
 xx := FSquareSize+ 2*BorderWidth + 1;

 if Fdirection = cbVert then     //if its in vertical mode
  begin
   width := xx; height := yy;
  end
  else      //vertical mode
   begin
    height := xx; width := yy;
   end;
//   Paint;
end;

procedure TColorDisplay.Clear;
begin
FColorPalette.Clear;
SetDimensions;
end;

constructor TColorDisplay.create(AOwner: TComponent);
begin
 inherited create(Aowner);
 FColorPalette:=TList.Create;
 FSquareSize := 10;
 BorderWidth := 4;
 FDirection := cbVert;
 FBorderLight := $ffffff;
 FBorderdark := $0;
 Parent:=AOwner as TWinControl;
 setdimensions;

end;

procedure TColorDisplay.setdirection(cbdir : TColorDisplaydir);
begin
 Fdirection := cbdir;
 setdimensions;
end;




procedure TColorDisplay.AddItem(const Value: TColor);
begin
  FColorPalette.Add(Pointer(Value));
  SetSquareSize(FSquareSize);
  setdimensions;
end;

destructor TColorDisplay.Destroy;
begin
  FColorPalette.Free;
  inherited Destroy;
end;

function TColorDisplay.GetColorPaletteCount: integer;
begin
result:=FColorPalette.Count;
end;

function TColorDisplay.GetColorSquare(Index : word) : TColorSquare;
var r,g,b : byte;
begin
 result.x := BorderWidth + Index * FSquareSize;
 result.y := BorderWidth;
 result.color := TColor(FColorPalette.Items[Index]);
end;




procedure TColorDisplay.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var index : Integer;
    Color: TColor;
begin
 if (x > BorderWidth) and (x < width-BorderWidth-1)
    and (y > BorderWidth) and (y < height-BorderWidth-1) then
  begin
     if FDirection = cbHor then
       Index := (x - BorderWidth) div FSquareSize
        else Index:= (y - BorderWidth) div FSquareSize;
  end
 else exit;

 FSelectedColor := GetColorSquare(Index).color;
 if assigned(FOnSelect) then
    FonSelect(self, Index, FSelectedColor);

end;

procedure TColorDisplay.Paint;
var h,i, x1, y1 : integer;
    cs : TColorSquare;
begin
inherited Paint;
  with canvas do
   begin
    pen.color := $0;
    pen.width := 1;
    for i := 0 to BorderWidth-1 do                  //borderpaint
     begin
      pen.color := Fborderlight;
      moveto(width-1-i,i);
      lineto(i,i);
      lineto(i,height-1-i);
      pen.color := FborderDark;
      lineto(width-1-i,height-1-i);
      lineto(width-1-i,i);
     end;
//--
    for i := 0 to FColorPalette.Count-1 do
     begin
      cs := GetColorSquare(i);
      if FDirection = CbVert then     //exchange x,y positions for vertical
       begin
        h := cs.x; cs.x := cs.y; cs.y := h;
       end;

      brush.color := cs.color;
      fillrect(rect(cs.x,cs.y,cs.x+FSquareSize,cs.y+FSquareSize));
     end;//for i
//--
    pen.color := $0;
    for i := 0 to ((width-2*BorderWidth) div FSquareSize) do //sep. lines
     begin
      x1 := BorderWidth + i * FSquareSize;
      moveto(x1,BorderWidth);
      lineto(x1,height - BorderWidth);
     end;
    for i := 0 to ((height-2*BorderWidth) div FSquareSize) do
     begin
      y1 := BorderWidth+ i * FSquareSize;
      moveto(BorderWidth,y1);
      lineto(width-BorderWidth,y1);
     end;
   end;
end;



procedure TColorDisplay.SetSquareSize(Size : byte);
begin
 if Size > 40 then Size := 40;
 if Size < 5 then Size := 5;
 FSquareSize := Size;
 SetDimensions;
end;



procedure TColorDisplay.setBorderwidth(w : byte);
begin
 if w > 10 then w := 10;
 BorderWidth := w;
 setdimensions;
end;

procedure TColorDisplay.SetItem(const Index: integer; NewColor: TColor);
begin
if((Index>=0)and(Index<FColorPalette.Count))then
 begin
  FColorPalette.Items[Index]:=Pointer(NewColor);
  Paint;
 end;
end;

procedure TColorDisplay.setBorderlight(c : longInt);
begin
 FBorderlight := c;
 paint;
end;

procedure TColorDisplay.setBorderdark(c : longInt);
begin
 FBorderdark := c;
 paint;
end;



end.



